package com.danitheskunk.skunkworks;

import org.lwjgl.assimp.AIMatrix4x4;

public final class Mat4f {
	private final double[] data;

	public static Mat4f IDENTITY = new Mat4f();

	public Mat4f() {
		data = new double[16];
		set(0, 0, 1.0);
		set(1, 1, 1.0);
		set(2, 2, 1.0);
		set(3, 3, 1.0);
	}

	public static Mat4f fromAiMat(AIMatrix4x4 aiMat) {
		var mat = new Mat4f();

		mat.set(0, 0, aiMat.a1());
		mat.set(1, 0, aiMat.a2());
		mat.set(2, 0, aiMat.a3());
		mat.set(3, 0, aiMat.a4());

		mat.set(0, 1, aiMat.b1());
		mat.set(1, 1, aiMat.b2());
		mat.set(2, 1, aiMat.b3());
		mat.set(3, 1, aiMat.b4());

		mat.set(0, 2, aiMat.c1());
		mat.set(1, 2, aiMat.c2());
		mat.set(2, 2, aiMat.c3());
		mat.set(3, 2, aiMat.c4());

		mat.set(0, 3, aiMat.d1());
		mat.set(1, 3, aiMat.d2());
		mat.set(2, 3, aiMat.d3());
		mat.set(3, 3, aiMat.d4());

		return mat;
	}

	public static Mat4f perspective(
		double fovy, double aspect, double zNear, double zFar
	) {
		var mat = new Mat4f();
		double tanHalfFovy = Math.tan(fovy / 2.0);
		mat.set(0, 0, 1.0 / (aspect * tanHalfFovy));
		mat.set(1, 1, 1.0 / tanHalfFovy);

		mat.set(2, 2, zFar / (zFar - zNear));
		mat.set(2, 3, 1.0);
		mat.set(3, 2, -(zFar * zNear) / (zFar - zNear));

		return mat;
	}

	public double[] asArray() {
		return data;
	}

	public float[] asFloatArray() {
		var arr = new float[16];
		for(int i = 0; i < 16; ++i) {
			arr[i] = (float) data[i];
		}
		return arr;
	}

	public double get(int x, int y) {
		return data[x + y * 4];
	}

	public void set(int x, int y, double val) {
		data[x + y * 4] = val;
	}
}
