package com.danitheskunk.skunkworks.backends.gl;

import com.danitheskunk.skunkworks.Vec2i;

public abstract class Filter {
	private final boolean resizing;
	protected Framebuffer input;
	protected Framebuffer output;

	public Filter(Framebuffer input, boolean resizing) {
		this.resizing = resizing;
		this.input = input;
		output = new Framebuffer(input.getSize());
	}

	public void apply() {
		output.bind();
		input.bindTexture();
		process();
	}

	public Framebuffer getOutput() {
		return output;
	}

	protected abstract void process();

	public void setInput(Framebuffer input) {
		this.input = input;
		if(resizing) {
			output.setSize(input.getSize());
		}
	}

	public void setOutputSize(Vec2i size) {
		if(!resizing) {
			throw new RuntimeException("this filter doesn't allow output resizing");
		}
		output.setSize(size);
	}
}
