package com.danitheskunk.skunkworks.backends.gl;

import com.danitheskunk.skunkworks.Recti;
import com.danitheskunk.skunkworks.Vec2i;
import com.danitheskunk.skunkworks.gfx.IPipeline;
import com.danitheskunk.skunkworks.gfx.IRenderContext;

import static org.lwjgl.opengl.GL11.*;

public class Pipeline2D extends BasePipeline<RenderContext, IRenderContext> implements IPipeline<IRenderContext> {
	private static final String fragmentSource = """
				#version 450
				layout(location = 1) in vec2 texCoord;
				layout(binding = 0) uniform sampler2D tex;
				layout(location = 3) uniform ivec2 texOffset;
				layout(location = 4) uniform ivec2 texSize;
				layout(location = 5) uniform vec4 tint;
				out vec4 color;
				void main() {
					//color = vec4(vec2(texCoord).x/1000.f, 0.2f, 0.7f, 1.0f);
					//color = texture(tex, texCoord);
					vec4 col = texelFetch(tex, ivec2(mod(texCoord, texSize)) + texOffset, 0);
					
					if(tint.a > 0.0) {
						color = col * tint;
					}
				}
		""";
	private static final String vertexSource = """
		#version 450
		layout(location = 0) in vec2 pos;
		layout(location = 1) in ivec2 texCoord;
		layout(location = 2) uniform vec2 windowSize;
		layout(location = 1) out vec2 out_texCoord;
		void main() {
			gl_Position = vec4(pos / windowSize * vec2(2.0f, -2.0f) + vec2(-1.0f, 1.0f), 0.0f, 1.0f);
			out_texCoord = texCoord;
		}
		""";
	private final TextureAtlas atlas;
	private boolean debug;
	private final Framebuffer framebuffer;

	public Pipeline2D(Framebuffer framebuffer, TextureAtlas atlas) {
		super(new Program(vertexSource, fragmentSource));
		renderContext = new RenderContext(program, framebuffer.getSize());
		this.atlas = atlas;
		this.framebuffer = framebuffer;
		debug = false;
	}

	@Override
	public void finishFrame() {
		if(debug) {
			renderContext.drawTextureRectangle(new Recti(Vec2i.ZERO, framebuffer.getSize()),
				atlas.getAtlasTexture(),
				false
			);
		}
		//Framebuffer.unbind();
	}

	public void setDebug(boolean debug) {
		this.debug = debug;
	}

	@Override
	public void startFrame() {
		use();
		atlas.bind();
		framebuffer.bind();
		var size = framebuffer.getSize();
		glViewport(0, 0, size.getX(), size.getY());
		atlas.update();
		glClear(GL_DEPTH_BUFFER_BIT);
		glDepthFunc(GL_ALWAYS);
		renderContext.preFrame();
	}
}
