package com.danitheskunk.skunkworks.backends.gl;

import com.danitheskunk.skunkworks.Mat4f;
import com.danitheskunk.skunkworks.Vec2i;
import com.danitheskunk.skunkworks.gfx.IPipeline;
import com.danitheskunk.skunkworks.gfx.threedee.IRenderContext3D;

import static org.lwjgl.opengl.GL11.*;
import static org.lwjgl.opengl.GL20.glUniformMatrix4fv;

public class Pipeline3D extends BasePipeline<RenderContext3D, IRenderContext3D> implements IPipeline<IRenderContext3D> {
	private static final String fragmentSource = """
		#version 450
		layout(location = 1) in vec2 texCoord;
		layout(binding = 0) uniform sampler2D tex;
		layout(location = 5) uniform vec4 tint;
		out vec4 color;
		void main() {
			//color = vec4(vec2(texCoord).x/1000.f, 0.2f, 0.7f, 1.0f);
			color = texture(tex, texCoord) * tint;
			//color = vec4(0.0, texCoord.xy, 1.0);
			//color = vec4(1.0, texCoord.x / 2, 1.0, 1.0);
		}
		""";
	private static final String vertexSource = """
		#version 450
		layout(location = 0) in vec3 pos;
		layout(location = 1) in vec2 texCoord;
		layout(location = 2) uniform mat4 projection;
		layout(location = 1) out vec2 out_texCoord;
		void main() {
			gl_Position = projection * vec4(pos, 1.0f);
			out_texCoord = texCoord;
		}
		""";

	private final Framebuffer framebuffer;
	private final TextureAtlas atlas;

	public Pipeline3D(Framebuffer framebuffer, TextureAtlas atlas) {
		super(new Program(vertexSource, fragmentSource));
		renderContext = new RenderContext3D(program);
		renderContext.setTextureAtlas(atlas);
		this.framebuffer = framebuffer;
		this.atlas = atlas;
	}

	@Override
	public void finishFrame() {
		//Framebuffer.unbind();
	}

	@Override
	public void startFrame() {
		use();
		atlas.bind();
		framebuffer.bind();
		var size = framebuffer.getSize();
		glViewport(0, 0, size.getX(), size.getY());
		atlas.update();
		glClear(GL_DEPTH_BUFFER_BIT);
		glDepthFunc(GL_LESS);

		//todo: stuff?
		var projection = Mat4f.perspective(Math.PI / 2, 16.0 / 9.0, 0.1, 100.0);
		glUniformMatrix4fv(program.getUniformLocation("projection"),
			true,
			projection.asFloatArray()
		);
		renderContext.preFrame();
	}
}
