package com.danitheskunk.skunkworks.gfx.threedee;

import com.danitheskunk.skunkworks.Mat4f;
import org.lwjgl.assimp.AIBone;

import java.util.ArrayList;
import java.util.List;

public class Bone {
	private String name;
	private List<VertexWeight> vertexWeights;
	private Mat4f offsetMatrix;

	public Bone() {
		vertexWeights = new ArrayList<>();
		offsetMatrix = Mat4f.IDENTITY;
	}

	public static Bone fromAIBone(AIBone aiBone) {
		var bone = new Bone();
		var weightBuffer = aiBone.mWeights();

		bone.name = aiBone.mName().dataString();

		while(weightBuffer.hasRemaining()) {
			var weight = VertexWeight.fromAiWeight(weightBuffer.get());
			bone.vertexWeights.add(weight);
		}

		bone.offsetMatrix = Mat4f.fromAiMat(aiBone.mOffsetMatrix());

		var node = aiBone.mNode();
		if(node.address() != 0) {
			System.out.println(node.mName().dataString());
		}

		return bone;
	}

	public VertexWeight getVertexWeight(int i) {
		return vertexWeights.get(i);
	}

	public int getVertexWeightCount() {
		return vertexWeights.size();
	}
}
