package com.danitheskunk.skunkworks.net;

import com.cedarsoftware.util.io.JsonReader;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.ProtocolException;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

public class HttpUtils {
	public static RequestBuilder request(String uriStr) {
		return new RequestBuilder(uriStr);
	}

	public static class RequestBuilder {
		private Map<String, String> customHeader;
		private Map<String, String> queryParameters;
		private String requestMethod;
		private String urlStr;

		private RequestBuilder(String urlStr) {
			customHeader = new HashMap<>();
			queryParameters = new HashMap<>();
			this.urlStr = urlStr;
			requestMethod = "GET";
		}

		private static String queryStrHelper(Map.Entry<String, String> entry) {
			return URLEncoder.encode(entry.getKey(), StandardCharsets.UTF_8) +
				"=" +
				URLEncoder.encode(entry.getValue(), StandardCharsets.UTF_8);
		}

		public RequestBuilder header(String key, String value) {
			customHeader.put(key, value);
			return this;
		}

		public RequestBuilder queryParameter(String key, String value) {
			queryParameters.put(key, value);
			return this;
		}

		public RequestBuilder requestMethod(String method) {
			requestMethod = method;
			return this;
		}

		public String run() {
			HttpURLConnection con;

			var paramStr = "";
			if(queryParameters.size() > 0) {
				paramStr = "?" + queryParameters.entrySet().stream().map(
					RequestBuilder::queryStrHelper).collect(Collectors.joining(
					"&"));
			}

			try {
				var url = new URL(urlStr + paramStr);
				con = (HttpURLConnection) url.openConnection();
			} catch(IOException e) {
				throw new RuntimeException(e);
			}

			try {
				con.setRequestMethod(requestMethod);
			} catch(ProtocolException e) {
				throw new RuntimeException(e);
			}

			for(var entry : customHeader.entrySet()) {
				con.setRequestProperty(entry.getKey(), entry.getValue());
				//System.out.printf("%s: %s\n", entry.getKey(), entry.getValue());
			}

			BufferedReader in;
			try {
				in = new BufferedReader(new InputStreamReader(con.getInputStream()));
			} catch(IOException e) {
				String inputLine;
				var ein = new BufferedReader(new InputStreamReader(con.getErrorStream()));
				try {
					while((inputLine = ein.readLine()) != null) {
						System.err.println(inputLine);
					}
				} catch(IOException ex) {
					throw new RuntimeException(ex);
				} throw new RuntimeException(e);
			}

			String inputLine;
			StringBuffer content = new StringBuffer();
			try {
				while((inputLine = in.readLine()) != null) {
					content.append(inputLine);
				}
				in.close();
			} catch(IOException e) {
				throw new RuntimeException(e);
			}
			return content.toString();
		}

		public java.util.Map runJSON() {
			return JsonReader.jsonToMaps(run());
		}

	}

}
