package com.danitheskunk.skunkworks.net;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.util.HashMap;
import java.util.Map;

public class IrcClient {
	private String buffer;
	private Map<String, IrcChannel> channels;
	private char[] chbuf;
	private InputStreamReader in;
	private BufferedWriter out;
	private Socket sock;

	public IrcClient(String hostname, int port, String nick) {
		this(hostname, port, nick, null);
	}

	public IrcClient(String hostname, int port, String nick, String pass) {
		buffer = "";
		//todo: increase size
		chbuf = new char[8];
		channels = new HashMap<>();
		try {
			sock = new Socket(hostname, port);
			in = new InputStreamReader(sock.getInputStream());
			out = new BufferedWriter(new OutputStreamWriter(sock.getOutputStream()));
		} catch(IOException e) {
			throw new RuntimeException(e);
		}

		if(pass != null) {
			send(new IrcMessage("PASS").addParam(pass));
		}
		send(new IrcMessage("NICK").addParam(nick));
	}

	public IrcChannel join(String channel) {
		send(new IrcMessage("JOIN").addParam(channel));
		var chan = new IrcChannel(this, channel);
		channels.put(channel, chan);
		return chan;
	}

	public void privmsg(String target, String msg) {
		send(new IrcMessage("PRIVMSG").addParam(target).addParam(msg));
	}

	private void processMessage(IrcMessage msg) {
		var cmd = msg.getCommand();
		if(cmd.equals("PING")) {
			send(new IrcMessage("PONG").addParam(msg.getParams().get(0)));
		} else if(cmd.equals("PRIVMSG")) {
			var params = msg.getParams();
			var target = params.get(0);
			var text = params.get(1);
			if(channels.containsKey(target)) {
				channels.get(target).processMessage(
					text,
					IrcHostmask.fromPrefix(msg.getPrefix())
				);
			} else {
				System.out.printf("got privmsg for unknown target '%s'\n",
					target
				);
			}
		}
		System.out.printf("IRC| %s", msg.toString());
	}

	public void run() {
		while(true) {
			tick();
			try {
				Thread.sleep(100);
			} catch(InterruptedException e) {
				throw new RuntimeException(e);
			}
		}
	}

	private void send(IrcMessage msg) {
		try {
			out.write(msg.toString());
			out.flush();
			//System.out.print(msg.toString());
		} catch(IOException e) {
			throw new RuntimeException(e);
		}
	}

	public void tick() {
		try {
			while(in.ready()) {
				//System.out.println("reading");
				int count = in.read(chbuf);
				var sb = new StringBuilder();
				sb.append(chbuf, 0, count);
				buffer += sb.toString();
				if(buffer.contains("\r\n")) {
					var sp = buffer.split("\r\n");
					buffer = sp.length == 1 ? "" : sp[1];
					processMessage(IrcMessage.fromString(sp[0]));
				}
			}
		} catch(IOException e) {
			throw new RuntimeException(e);
		}
	}
}
