package com.danitheskunk.skunkworks.net;

import java.util.ArrayList;
import java.util.List;

public class IrcMessage {
	private String command;
	private List<String> params;
	private String prefix;

	public IrcMessage() {
		params = new ArrayList<>();
	}

	public IrcMessage(String cmd) {
		this();
		command = cmd;
	}

	public static IrcMessage fromString(String str) {
		var irc = new IrcMessage();

		if(str.startsWith(":")) {
			var sp = str.split(" ", 2);
			irc.setPrefix(sp[0].substring(1));
			str = sp[1];
		}

		if(str.contains(" ")) { //has parameters
			var sp = str.split(" ", 2);
			irc.setCommand(sp[0]);
			str = sp[1];
			while(true) {
				if(str.startsWith(":")) { //last parameter with :
					irc.addParam(str.substring(1).strip());
					break;
				}
				if(str.contains(" ")) { //not last parameter
					sp = str.split(" ", 2);
					irc.addParam(sp[0]);
					str = sp[1];
				} else { //last parameter
					irc.addParam(str.strip());
					break;
				}
			}
		} else {
			irc.setCommand(str.strip());
		}

		return irc;
	}

	public IrcMessage addParam(String param) {
		params.add(param);
		return this;
	}

	public String getCommand() {
		return command;
	}

	public List<String> getParams() {
		return params;
	}

	public String getPrefix() {
		return prefix;
	}

	public IrcMessage setCommand(String command) {
		this.command = command;
		return this;
	}

	public IrcMessage setPrefix(String prefix) {
		this.prefix = prefix;
		return this;
	}

	@Override
	public String toString() {
		var sb = new StringBuilder();
		if(prefix != null) {
			sb.append(":");
			sb.append(prefix);
			sb.append(" ");
		}
		sb.append(command);
		if(params.size() > 0) {
			sb.append(" ");
		}
		for(int i = 0; i < params.size(); ++i) {
			if(i == params.size() - 1) {
				if(params.get(i).contains(" ")) {
					sb.append(":");
				}
				sb.append(params.get(i));
			} else {
				sb.append(params.get(i));
				sb.append(" ");
			}
		}
		sb.append("\r\n");
		return sb.toString();
	}
}
